/*
    cobbler-enroll - Cobbler enrollment tool
    Copyright (C) 2011 Canonical Ltd.

    Authors: Dave Walker (Daviey) <DaveWalker@ubuntu.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

static int fault_occurred (xmlrpc_env * const envP) {
  if (!envP->fault_occurred)
    return 0;
  fprintf (stderr, "ERROR: %s (%d)\n", envP->fault_string, envP->fault_code);
  return 1;
}

int init_xmlrpc_env(struct cobbler_client *enroller) {
  if (debug) {
    printf("[DEBUG] Initializing xmlrpc enviornment.\n");
  }
  xmlrpc_env_init(&enroller->env);
  if(fault_occurred(&enroller->env) != 0)
    return 1;
  xmlrpc_client_setup_global_const(&enroller->env);
  return fault_occurred(&enroller->env);
}

int init_xmlrpc_client(struct cobbler_client *enroller) {
  if (debug)
    printf("[DEBUG] Initializing xmlrpc client.\n");
/*  xmlrpc_client_init2(&enroller->env,
                      XMLRPC_CLIENT_NO_FLAGS,
                      NAME, VERSION, NULL, 0);
  if (!fault_occured(&enroller->env))
    return 1;*/
  xmlrpc_client_create(&enroller->env,
                       XMLRPC_CLIENT_NO_FLAGS,
                       NAME, VERSION, 0, 0,
                       &enroller->client);
  return fault_occurred(&enroller->env);
}

int make_xmlrpc_call(struct cobbler_client *enroller,
                     char *method,
                     xmlrpc_value *arguments) {
  if (debug)
    printf("[DEBUG] Making xmlrpc call for method: %s\n", method);
  xmlrpc_server_info *serverinfo;
  serverinfo = xmlrpc_server_info_new(&enroller->env, enroller->serverUrl);
  xmlrpc_client_call2(&enroller->env,
                      enroller->client,
                      serverinfo,
                      method,
                      arguments,
                      &enroller->resultP);
  /* TODO: This causes error, find out why.
  xmlrpc_server_info_free(serverinfo); */
  return fault_occurred(&enroller->env);
}


int cobbler_login(struct cobbler_client *enroller) {
  if (debug)
    printf("[DEBUG] cobbler_login()\n");
  int rc = 0;
  xmlrpc_value *argument_array;
  xmlrpc_value *username;
  xmlrpc_value *password;

  argument_array = xmlrpc_array_new(&enroller->env);
  username = xmlrpc_string_new(&enroller->env, enroller->username);
  password = xmlrpc_string_new(&enroller->env, enroller->password);
  xmlrpc_array_append_item(&enroller->env, argument_array, username);
  xmlrpc_array_append_item(&enroller->env, argument_array, password);
  xmlrpc_DECREF(username);
  xmlrpc_DECREF(password);
  rc = make_xmlrpc_call(enroller, "login", argument_array);
  if (rc != 0)
    goto out;
  xmlrpc_read_string(&enroller->env, enroller->resultP, &enroller->token);
  if ((rc = fault_occurred(&enroller->env)) != 0)
    goto out;
  if (debug)
    printf("[DEBUG] The token is %s\n", enroller->token);
out:
  xmlrpc_DECREF(argument_array);
  if (debug)
    printf("[DEBUG] Login complete. rc=%d\n", rc);
  return rc;
}

int cobbler_register_system(struct cobbler_client *enroller) {
  if (debug)
    printf("[DEBUG] cobbler_register_system()\n");
  int rc = 0;
  xmlrpc_value *token = xmlrpc_string_new(&enroller->env, enroller->token);
  xmlrpc_value *name = xmlrpc_string_new(&enroller->env, enroller->name);
  xmlrpc_value *profile = xmlrpc_string_new(&enroller->env, enroller->profile);
  xmlrpc_value *argument_array;
  argument_array = xmlrpc_array_new(&enroller->env);
  xmlrpc_array_append_item(&enroller->env, argument_array, token);
  if ((rc = make_xmlrpc_call(enroller, "new_system", argument_array)) != 0)
    goto out;

  xmlrpc_read_string(&enroller->env, enroller->resultP, &enroller->system);
  xmlrpc_value *system = xmlrpc_string_new(&enroller->env, enroller->system);
  printf("[DEBUG] Registered new system: rc: %d system: %s\n", rc, enroller->system);

  argument_array = xmlrpc_array_new(&enroller->env);
  xmlrpc_array_append_item(&enroller->env, argument_array, system);
  xmlrpc_array_append_item(&enroller->env, argument_array,
                           xmlrpc_string_new(&enroller->env, "name"));
  xmlrpc_array_append_item(&enroller->env, argument_array, name);
  xmlrpc_array_append_item(&enroller->env, argument_array, token);
  if ((rc = make_xmlrpc_call(enroller, "modify_system", argument_array)) != 0)
    goto out;
  printf("[DEBUG] Modify system name: %d\n", rc);

  argument_array = xmlrpc_array_new(&enroller->env);
  xmlrpc_array_append_item(&enroller->env, argument_array, system);
  xmlrpc_array_append_item(&enroller->env, argument_array,
                           xmlrpc_string_new(&enroller->env, "profile"));
  xmlrpc_array_append_item(&enroller->env, argument_array, profile);
  xmlrpc_array_append_item(&enroller->env, argument_array, token);
  if ((rc = make_xmlrpc_call(enroller, "modify_system", argument_array)) != 0)
    goto out;
  printf("[DEBUG] Modify system profile: %d\n", rc);

out:
  xmlrpc_DECREF(token);
  xmlrpc_DECREF(name);
  xmlrpc_DECREF(profile);
  xmlrpc_DECREF(system);
  xmlrpc_DECREF(argument_array);
  return rc;
}

int cobbler_save_system(struct cobbler_client *enroller) {
  if (debug)
    printf("[DEBUG] cobbler_save_system()\n");
  int rc = 0;
  xmlrpc_value *argument_array;
  argument_array = xmlrpc_array_new(&enroller->env);
  xmlrpc_value *token = xmlrpc_string_new(&enroller->env, enroller->token);
  xmlrpc_value *system = xmlrpc_string_new(&enroller->env, enroller->system);
  xmlrpc_array_append_item(&enroller->env, argument_array, system);
  xmlrpc_array_append_item(&enroller->env, argument_array, token);
  if ((rc = make_xmlrpc_call(enroller, "save_system", argument_array)) != 0)
    goto out;
  printf("[DEBUG] System saved.");
out:
  xmlrpc_DECREF(token);
  xmlrpc_DECREF(system);
  xmlrpc_DECREF(argument_array);
  return rc;
}

